############################################################################
# SendSerial
# Copyright (c) 2006 Factorycode <info@factorycode.com>
 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
# CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
# ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
# COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
# SOFTWARE IS DISCLAIMED.
############################################################################

package send_serial;

use 5.006001;
use strict;
use warnings;
use Win32::API;
use Win32::SerialPort;

my $serialDev;
my $IOinit;

require Exporter;
our @ISA         = qw(Exporter);
our %EXPORT_TAGS = (
  'all' => [
    qw(
      initSIO
      closeSIO
      sendString
	  readChar
	  sendAndCheckEcho
      )
  ]
);
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT    = qw(
);
our $VERSION = '0.1';


sub sendString {
  my $output = shift;
  # Clear any buffered characters by sending a 'clear buffer' command.
  $serialDev->purge_all; #Clear receive & transmit buffer
  $serialDev->read_done(1);
  $serialDev->write($output);
}

sub readChar {
    my $input = "";
    my $return_result = "";

    $input = $serialDev->read(1);
    if ((defined $input) &&  ($input ne '')) # If $input equals "0", Perl thinks it is FALSE in a conditional... amazing...
    {
      # Is it a non-escaped 'handshake' character?
      $return_result= $input;
    }
    return $return_result; 
}

sub sendAndCheckEcho {
  my $caller = shift;
  my $output = shift;
  my $timeout = 5000;

  my $keepTrying  = 10;
  my $input       = "";
  my $done        = 0;
  my $result      = "";

  sendString($output);
  my $timeoutWhen = $timeout + $serialDev->get_tick_count; # Start timeout after transmission
  
  # It might fail, so we'll give it a few chances to do things right...
  while ( ($keepTrying > 0) && ($done == 0)) {
    
    $result = $result . readChar();
    if ( $serialDev->get_tick_count > $timeoutWhen ) {
      warn $serialDev->get_tick_count . " SIO timeout when sending $output (reply buffer: $result), retrying.";
      $keepTrying--;
      $timeoutWhen = $timeout + $serialDev->get_tick_count;
      sendString($output);
      $result = "";
    }

    if ( length($result) == length($output) ) {
      if ( substr( $result, 0, length($output) ) eq $output )
      {
        $done = 1;
	  }
      else
      {
        my $message = "Unknown characters received: \$output\, when waiting for echo response. Character discarded.";
        $result = "";
      }
    }
   
    # Tired of trying? Just give up...
    if (($keepTrying == 0) && ($done==0))
    {
      print "Maximum number of retries (timeouts and/or checksum failures) exceeded when sending command.";
    }
  }
  return $result;
}

sub initSIO {
  my $caller = shift;
  my $properties = shift;

  my $device    = $properties->getProperty('SIO::device', 'DIE');
  my $baudrate  = $properties->getProperty('SIO::baudrate', 'DIE');
  my $parity    = $properties->getProperty('SIO::parity', 'DIE');
  my $databits  = $properties->getProperty('SIO::databits', 'DIE');
  my $stopbits  = $properties->getProperty('SIO::stopbits', 'DIE');
  my $handshake = $properties->getProperty('SIO::handshake', 'DIE');

  $serialDev = new Win32::SerialPort( $device, 1 )
    || die ("Error: Can't open $device.\n" . "Possible cause: the port/device does not exist or is busy/locked by another process.");
  
  $serialDev->baudrate($baudrate);
  $serialDev->parity($parity);
  $serialDev->databits($databits);
  $serialDev->stopbits($stopbits);
  $serialDev->handshake($handshake);
  
  #Windows serial ports must have everything set up since there is no default settings, they just inherit from the last program using the COM-port.
  $serialDev->xon_limit(100);
  $serialDev->xoff_limit(100);
  $serialDev->xon_char(0x11);
  $serialDev->xoff_char(0x13);
  $serialDev->eof_char(0x0);
  $serialDev->event_char(0x0);
  $serialDev->error_char(0);
  $serialDev->buffers(4096, 4096);
  $serialDev->read_interval(200);    # max time between read char (milliseconds)
  $serialDev->read_char_time(5);     # avg time between read char
  $serialDev->read_const_time(200);  # total = (avg * bytes) + const 
  $serialDev->write_char_time(5);
  $serialDev->write_const_time(100);

  $serialDev->write_settings || undef $serialDev;

  $serialDev->purge_all;

  #$logger->info("Initialize serial I/O: $device, $baudrate, $parity, $databits, $stopbits, $handshake.");
  $IOinit = 1;
  return $IOinit;
}

sub closeSIO {
  $serialDev->close;
}

